﻿using System.Diagnostics;
using System.Linq;
using System.Threading;
using CommonBehaviors.Actions;
using LazyBoxer.Helpers;
using Styx;
using Styx.CommonBot;
using Styx.Pathing;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Action = Styx.TreeSharp.Action;

namespace LazyBoxer.Behaviors
{
    static class Common
    {
        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private static WoWUnit CT { get { return Me.CurrentTarget; } }

        public static WoWUnit HostileMobInOurPath(double maxDistanceAway = 30, int degrees = 30, int ourPullDistance = 35)
        {
            return ObjectManager.GetObjectsOfTypeFast<WoWUnit>().Where(u => u.IsHostile && u.Distance < maxDistanceAway && u.IsAlive && !u.TaggedByOther && u.Distance < ourPullDistance && Me.IsSafelyFacing(u, degrees) && u.InLineOfSpellSight).OrderBy(u => u.Distance).FirstOrDefault();
        }

        public static bool IsGameStable()
        {
            return StyxWoW.IsInGame && Me != null && Me.IsValid;
        }

        #region Mounting

        /// <summary>
        /// Wait for us to stop moving. Max time is our ping
        /// </summary>
        public static void WaitForStop()
        {
            WoWMovement.MoveStop(WoWMovement.MovementDirection.All);
            WoWMovement.MoveStop();
            Navigator.PlayerMover.MoveStop();

            do { StyxWoW.SleepForLagDuration(); }
            while (Common.IsGameStable() && Me.CurrentHealth > 1 && Me.IsMoving);
        }

        /// <summary>
        /// Wait for us to mount up. Max time is our ping
        /// </summary>
        /*
        public static void WaitForMount()
        {
            if (Me.Combat || Me.IsIndoors) return;

            WaitForStop();

            Stopwatch timeOut = new Stopwatch();
            timeOut.Start();

            if (!Mount.Mounts.Any() || !Mount.CanMount()) return;

            TreeRoot.StatusText = "Leader has mounted so we'll do the same";

            Mount.MountUp(LazyLocationRetriever);
            //StyxWoW.SleepForLagDuration();

            //_followDistance = Setting.Instance.FollowDistance;
            //Setting.Instance.FollowDistance = 5;

            while (Common.IsGameStable() && Me.CurrentHealth > 1 && Me.IsCasting)
            {
                Thread.Sleep(75);
            }

        }
         */

        public static Composite MountUp()
        {
            return
                new Decorator(ret => !Me.Mounted && !Me.Combat && !Me.IsIndoors && Mount.CanMount(),
                              new Sequence(
                                  // Stop moving
                                  new DecoratorContinue(ret=> Me.IsMoving, Movement.CreateEnsureMovementStoppedBehavior()),
                                  new WaitContinue(2, ret => !Me.IsMoving, new ActionAlwaysSucceed()),

                                  // Cast mount
                                  new Action(context => TreeRoot.StatusText = "Leader has mounted so we'll do the same"),
                                  new Action(context => Mount.MountUp(LazyLocationRetriever)),
                                  
                                  // wait for casting to start
                                  new Wait(2, ret => Me.IsCasting, new ActionAlwaysSucceed())

                                  )
                    );
        }

        public static Composite Dismount()
        {
            return
                new Decorator(ret => Me.Mounted,
                    new Sequence(
                        // Cast mount
                        new Action(context => TreeRoot.StatusText = "Leader has dismounted so we'll do the same"),
                        new Action(context => Lua.DoString("Dismount()"))

                    )
                );
        }
        

        /// <summary>
        /// Wait for us to dismount. Max time is our ping
        /// </summary>
        /*
        public static void WaitForDismount()
        {
            while (Common.IsGameStable() && Me.CurrentHealth > 1 && Me.Mounted)
            {
                TreeRoot.StatusText = "Dismounting";
                Lua.DoString("Dismount()");
                //StyxWoW.SleepForLagDuration();
                //Setting.Instance.FollowDistance = _followDistance;
            }
        }
         */

        internal static WoWPoint LazyLocationRetriever()
        {
            return WoWPoint.Empty;
        }

        public static bool NeedToMount()
        {
            return !Me.Mounted && FollowTheLeader.Leader != null && (FollowTheLeader.Leader.Distance > 60 || FollowTheLeader.Leader.Mounted) && Me.IsOutdoors && Mount.CanMount();
        }

        public static bool NeedToDismount()
        {
            return Me.Mounted && FollowTheLeader.Leader != null && FollowTheLeader.Leader.Distance <= 30 && !FollowTheLeader.Leader.Mounted;
        }
        #endregion
    }
}
