﻿using System;
using System.Linq;
using CommonBehaviors.Actions;
using JetBrains.Annotations;
using LazyBoxer.Helpers;
using Styx;
using Styx.CommonBot;
using Styx.Pathing;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Action = Styx.TreeSharp.Action;

namespace LazyBoxer.Behaviors
{
    static class FollowTheLeader
    {
        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private static WoWUnit CT { get { return Me.CurrentTarget; } }

        public static bool BotMovement { get; set; }
        public static bool WaitingHere { get; set; }

        //public static bool ForceAttacking { get; set; }
        //public static bool SellingAtVendor { get; set; }

        private static WoWPoint _lastDest;

        static void _leader_OnInvalidate()
        {
            //_leader.OnInvalidate -= _leader_OnInvalidate;
            _leader = null;
        }

        public static WoWUnit _leader;
        public static string FollowLeaderName = "";
        public static WoWUnit Leader
        {
            get
            {
                if (Setting.Instance.FollowLeader == "[Never]") return null;
                if (Setting.Instance.FollowLeader == "[None]") return null;

                //if (Setting.Instance.FollowLeader.Contains("None")) return null;
                // return our cached leader if its been less than 4 seconds
                //if (_leader != null && _leader.IsValid && _leader.IsAlive && !Timers.AutoExpire("Find a tank", 4000)) return _leader;


                // We've specifically identified a player to follow
                if (!Setting.Instance.FollowLeader.Contains("Never") && !Setting.Instance.FollowLeader.Contains("None") && !Setting.Instance.FollowLeader.Contains("Automatic"))
                {
                    // If the leader has not changed and they are valid return our old leader
                    if (_leader != null && _leader.IsValid && _leader.IsAlive && _leader.Name == Setting.Instance.FollowLeader) return _leader;

                    // Find a suitable leader
                    if (_leader == null || !_leader.IsValid || !_leader.IsAlive || _leader.Name != Setting.Instance.FollowLeader)
                    {
                        WoWUnit unit = ObjectManager.GetObjectsOfType<WoWPlayer>(false, false).FirstOrDefault(u => u.IsAlive && u.Distance < 100 && u.Name == Setting.Instance.FollowLeader);
                        if (unit != null)
                        {
                            TreeRoot.StatusText = "Leader choosen by name: " + unit.Name;
                            //_followLeaderName = unit.Name;
                            RaFHelper.SetLeader(unit.Guid);
                            _leader = unit;

                            //_leader.OnInvalidate += _leader_OnInvalidate;
                            return unit;
                        }
                    }
                }

                // We're here because we don't have a suitable leader. 
                if (Me.GroupInfo.IsInRaid && (!Setting.Instance.FollowLeader.Contains("Never") && !Setting.Instance.FollowLeader.Contains("None")))
                {
                    // If the leader has not changed and they are valid return our old leader
                    if (_leader != null && _leader.IsValid && _leader.IsAlive && !Timers.AutoExpire("Find a tank", 4000)) return _leader;

                    for (int i = 1; i < Me.GroupInfo.RaidMembers.Count(); i++)
                    {
                        string role = Lua.GetReturnVal<string>(String.Format("return UnitGroupRolesAssigned('raid{0}')", i), 0);
                        if (role != "TANK") continue;

                        WoWUnit possibleLeader = ObjectManager.GetObjectByGuid<WoWPlayer>(Me.GetRaidMemberGuid(i - 1));

                        // Make sure they are valid and alive
                        if (possibleLeader != null && !possibleLeader.IsAlive) continue;
                        if (possibleLeader != null)
                        {
                            _leader = possibleLeader;
                            //_followLeaderName = possibleLeader.Name;
                            RaFHelper.SetLeader(_leader.Guid);
                            TreeRoot.StatusText = "Leader choosen by Raid tank role: " + possibleLeader.Name;

                            //_leader.OnInvalidate += _leader_OnInvalidate;
                            return _leader;
                        }
                        break;
                    }
                }

                if ((Me.GroupInfo.IsInParty || Me.GroupInfo.IsInLfgParty) && (!Setting.Instance.FollowLeader.Contains("Never") && !Setting.Instance.FollowLeader.Contains("None")))
                {
                    // If the leader has not changed and they are valid return our old leader
                    if (_leader != null && _leader.IsValid && _leader.IsAlive && !Timers.AutoExpire("Find a tank", 4000)) return _leader;

                    for (int i = 1; i < Me.GroupInfo.PartyMembers.Count(); i++)
                    {
                        string role = Lua.GetReturnVal<string>(String.Format("return UnitGroupRolesAssigned('party{0}')", i), 0);
                        if (role != "TANK") continue;

                        WoWUnit possibleLeader = ObjectManager.GetObjectByGuid<WoWPlayer>(Me.GetRaidMemberGuid(i - 1));

                        // Make sure they are valid and alive
                        if (possibleLeader != null && !possibleLeader.IsAlive) continue;
                        if (possibleLeader != null)
                        {
                            _leader = possibleLeader;
                            //_followLeaderName = possibleLeader.Name;
                            TreeRoot.StatusText = "Leader choosen by Party tank role: " + possibleLeader.Name;
                            RaFHelper.SetLeader(_leader.Guid);
                            _leader.OnInvalidate += _leader_OnInvalidate;
                            return _leader;
                        }
                        break;
                    }
                }

                // If we are here then just take the first alive player we can find
                try
                {
                    WoWPlayer lastResortLeader = ObjectManager.GetObjectsOfType<WoWPlayer>(false, false).FirstOrDefault(p => p.IsAlive);
                    TreeRoot.StatusText = "Leader choosen by last resort: " + lastResortLeader.Name;
                    _leader = lastResortLeader;
                    RaFHelper.SetLeader(_leader.Guid);
                    return lastResortLeader;
                }
                catch (Exception e)
                {
                    Console.WriteLine(e);
                }

                return null;


                /*
                                return null;
                */
            }
        }

        public static Composite CreateFollowTheLeaderBehavior()
        {
            return
                // Follow leader - if we're not doing a lot of other stuff
                new Decorator(ret =>
                    Looting.Lootable == null &&
                    Skinning.SkinnableMob == null &&
                    Mining.OreHarvestable == null &&
                    Herbalism.HerbHarvestable == null &&
                    InteractableObjects.QuestItemHarvestable == null &&
                    Leader != null &&
                    !ForceAttack.ForceAttacking &&
                    !Vendoring.SellingAtVendor &&
                    !Me.HasAura("Eat") && !Me.HasAura("Drink") &&
                    ((Me.Combat && Setting.Instance.CombatFollow.OK()) || !Me.Combat),

                    new PrioritySelector(CreateFollowBehavior()));
        }

        [UsedImplicitly]
        private static bool _isInitialized;

        public static int FollowDistance { get { return Setting.Instance.CombatFollow.OK() ? Setting.Instance.CombatFollowDistance : Setting.Instance.FollowDistance; } }

        public static Composite CreateFollowBehavior()
        {
            return new PrioritySelector(

                // Reasons NOT to follow leader
                new Decorator(ret => Setting.Instance.FollowLeader.Contains("Never") || Setting.Instance.FollowLeader.Contains("None"), new ActionAlwaysSucceed()),
                new Decorator(ret => Misc.IsUserControllingMovement, new ActionAlwaysSucceed()),
                new Decorator(ret => WaitingHere, new ActionAlwaysSucceed()),
                new Decorator(ret => Leader == null, new ActionAlwaysSucceed()),
                new Decorator(ret => Me.IsDead || Me.IsGhost || Me.CurrentHealth <= 1, new ActionAlwaysSucceed()),

                // Mount up and Dismount as required
                new Decorator(ret => Common.NeedToMount(), Common.MountUp()),
                new Decorator(ret => Common.NeedToDismount(), Common.Dismount()),//new Action(delegate { Common.WaitForDismount(); })),


                // Finally just move to our follow target
                //new Decorator(ret => (FollowTarget != null && FollowTarget.Distance > FollowDistance || (FollowTarget != null && !FollowTarget.InLineOfSpellSight)) && (Navigator.CanNavigateFully(Me.WorldLocation, FollowTarget.WorldLocation) || !Me.IsFlying || !Me.IsSwimming),
                new Decorator(ret => (Leader != null && Leader.Distance > FollowDistance || (Leader != null && !Leader.InLineOfSpellSight)) && (Navigator.CanNavigateFully(Me.WorldLocation, Leader.WorldLocation) || !Me.IsFlying || !Me.IsSwimming),
                    new Action(context =>
                    {
                        WoWPoint ftLocation = Leader.WorldLocation;
                        if (ftLocation.DistanceSqr(_lastDest) > 1 || !Me.IsMoving)
                        {
                            _lastDest = ftLocation;
                            WoWPoint newPoint = WoWMovement.CalculatePointFrom(ftLocation, (float)0.85 * FollowDistance);

                            if (Me.Mounted || Me.IsFlying || Me.IsSwimming && (Leader.IsFlying || Leader.IsSwimming))
                            {
                                TreeRoot.StatusText = "Flying / Swimming to our leader";
                                Flightor.MoveTo(newPoint);
                            }
                            else
                            {
                                TreeRoot.StatusText = "Following our leader";
                                Navigator.MoveTo(newPoint);
                            }

                            BotMovement = true;
                        }

                        return RunStatus.Success;
                    })),


                // Do we need to stop moving?
                new Decorator(ret => Me.IsMoving && BotMovement,
                    new Action(delegate
                    {
                        BotMovement = false;
                        while (Common.IsGameStable() && Me.IsMoving)
                        {
                            TreeRoot.StatusText = "";
                            WoWMovement.MoveStop();
                            if (Me.IsMoving) { StyxWoW.SleepForLagDuration(); }
                        }

                        return RunStatus.Success;
                    }))



                );
        }
    }
}
