﻿using System;
using System.Linq;
using CommonBehaviors.Actions;
using LazyBoxer.Helpers;
using Styx;
using Styx.CommonBot;
using Styx.CommonBot.POI;
using Styx.CommonBot.Routines;
using Styx.Pathing;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Action = Styx.TreeSharp.Action;

namespace LazyBoxer.Behaviors
{
    static class Skinning
    {
        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private static WoWUnit CT { get { return Me.CurrentTarget; } }

        public static Composite CreateSkinning()
        {
            return
                new PrioritySelector(

                    new Throttle(TimeSpan.FromSeconds(2),
                                 new Decorator(ret => Setting.Instance.InteractSkinning.OK() && !Misc.IsUserControllingMovement && !Me.IsCasting && !Me.IsChanneling && !Me.Combat && !Me.Looting && SkinnableMob != null,

                                     new Sequence(
                                         new Decorator(ret => { { Log.Info("Skinnable mob is visible: " + SkinnableMob.Name); return true; } }, new ActionAlwaysSucceed()),
                                         new Action(context => Log.Info("==== 1")),

                                         // Move closer to the object
                                         new DecoratorContinue(ret => SkinnableMob.Distance > SkinnableMob.InteractRange * 0.85f && !Me.Combat,
                                             new Sequence(

                                                 // Move closer to the object
                                                 Movement.CreateMoveToLocationBehavior(ret => SkinnableMob.WorldLocation, true, ret => SkinnableMob.InteractRange * 0.55f),

                                                 // wait for up to 2 seconds for us to start moving towards the object
                                                 new Wait(2, ret => Me.IsMoving, new ActionAlwaysSucceed())
                                                 )),

                                                 // Hostile mob in our path and we've flagged it for death!
                                        new DecoratorContinue(ret => Common.HostileMobInOurPath(SkinnableMob.Distance + 7) != null && Common.HostileMobInOurPath(SkinnableMob.Distance + 5).Distance < 30 && Me.GotTarget && BotPoi.Current.Type == PoiType.Kill,
                                          new Sequence(

                                              new Action(context => Log.Info("==== hostile mob in our path, we're going to kill it")),

                                              // Stop moving
                                              new DecoratorContinue(ret => Me.IsMoving, Movement.CreateEnsureMovementStoppedBehavior()),
                                              new WaitContinue(2, ret => !Me.IsMoving, new ActionAlwaysSucceed()),

                                              // Clear navigator so we don't jump
                                              new Action(context => Navigator.Clear()),

                                              // Call the CC pull logic      
                                              new Decorator(ret => RoutineManager.Current.PullBehavior != null, RoutineManager.Current.PullBehavior),

                                              // Call the CC combat logic as some CCs may not have pull logic 
                                              new Decorator(ret => RoutineManager.Current.PullBehavior != null, RoutineManager.Current.CombatBehavior)
                                          )),

                                           // Attack mobs in front of us if we're moving to harvest
                                           new DecoratorContinue(ret => Me.IsMoving && Common.HostileMobInOurPath(SkinnableMob.Distance + 7) != null && !Me.IsCasting && SkinnableMob.Distance > SkinnableMob.InteractRange * 0.85f,
                                               new Throttle(2,
                                                   new Sequence(
                                                      new Action(context =>
                                                      {
                                                          Log.Info(string.Format("Hostile mob in our path, {0}, we need to kill it", Common.HostileMobInOurPath(SkinnableMob.Distance + 5).Name));
                                                          Common.HostileMobInOurPath(SkinnableMob.Distance + 5).Target();
                                                          BotPoi.Current = new BotPoi(Common.HostileMobInOurPath(SkinnableMob.Distance + 5), PoiType.Kill);

                                                      })
                                                  )
                                              )),


                                              // Dismount if we're mounted
                                         new DecoratorContinue(ret=> !Me.Combat && !Me.Mounted && SkinnableMob.Distance < SkinnableMob.InteractRange * 3,
                                                       Common.Dismount()),

                                         new Decorator(ret => !Me.Combat && SkinnableMob.Distance <= SkinnableMob.InteractRange * 0.85f && SkinnableMob.IsValid,
                                             new Sequence(

                                                 // Do some bail out checks
                                                 new Action(c => !SkinnableMob.IsValid ? RunStatus.Failure : RunStatus.Success),
                                                 new Action(c => Me.Combat ? RunStatus.Failure : RunStatus.Success),

                                                 // if we're moving then and we're in interact range then stop
                                                 new DecoratorContinue(ret => SkinnableMob.Distance <= SkinnableMob.InteractRange * 0.85f && Me.IsMoving,
                                                     new PrioritySelector(Movement.CreateEnsureMovementStoppedBehavior())),

                                                 // Make sure we're not moving. Just in case
                                                 new WaitContinue(2, ret => !Me.IsMoving, new ActionAlwaysSucceed()),

                                                 // Finally interact with it
                                                 new Action(c => SkinnableMob.Interact()),

                                                 // wait for casting to start, max 2 seconds
                                                 new Wait(2, ret => Me.IsCasting, new ActionAlwaysSucceed()),

                                                 // clear all navigation as we tend to jump after we've looted
                                                 new Action(c => { Navigator.Clear(); return RunStatus.Success; }),

                                                 // wait while we are casting. Max 10 seconds
                                                 new WaitContinue(10, ret => !Me.IsCasting, new ActionAlwaysSucceed()),

                                                 // Casting has finished, but wait 2 more seconds for looting to occur
                                                 new WaitContinue(1, ret => false, new ActionAlwaysSucceed()),
                                                 new Wait(1, ret => Me.Looting, new ActionAlwaysSucceed()),
                                                 new Action(c => { if (Me.Looting) Blacklist.Add(SkinnableMob.Guid, BlacklistFlags.Loot, TimeSpan.FromMinutes(2)); }),
                                                 new WaitContinue(5, ret => !Me.Looting, new ActionAlwaysSucceed()),

                                                 // Black list so we don't try to loot again for min 1 minute
                                                 new Action(c => Blacklist.Add(SkinnableMob.Guid, BlacklistFlags.Loot, TimeSpan.FromMinutes(2))),

                                                 // clear all navigation as we tend to jump after we've looted
                                                 new Action(c => { Navigator.Clear(); return RunStatus.Success; })

                                                 ))


                                         )




                                     )
                        ));
        }


        public static WoWUnit SkinnableMob
        {
            get
            {
                if (Setting.Instance.InteractSkinning.OK())
                {
                    return
                        ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                                     .Where(o => o.Distance < Setting.Instance.InteractMaxRange && o.Skinnable
                                         && o.CanSkin
                                         && !Blacklist.Contains(o.Guid, BlacklistFlags.Loot)
                                         )
                                     .OrderBy(o => o.Distance)
                                     .FirstOrDefault();
                }

                return null;
            }
        }

    }
}
